@@ -54,12 +54,14 @@ rm -f CMakeCache.txt
 cmake \
 ${RPATH} \
 -D CATALYST_BUILD_SHARED_LIBS:BOOL=${SHARED} \
+-D CMAKE_POSITION_INDEPENDENT_CODE:BOOL=ON \
 -D CMAKE_PREFIX_PATH:PATH=${INSTALL_PATH}/lib \
 -D CMAKE_INSTALL_PREFIX:PATH=${INSTALL_PATH} \
 -D CMAKE_INSTALL_LIBDIR:PATH=lib \
 -D CATALYST_BUILD_STUB_IMPLEMENTATION:BOOL=ON \
 -D CATALYST_USE_MPI:BOOL=${MPI} \
 -D CATALYST_BUILD_TESTING:BOOL=OFF \
+-D CATALYST_WRAP_PYTHON:BOOL=OFF \
 $EXTRA_ARGS \
 ..
 
