@@ -25,31 +25,31 @@ function check_exec()
 
 function check_valid()
 {
-    if [ "${!1}" == "YES" ] || [ "${!1}" == "ON" ]; then
+    if [ "${!1}" == "YES" ] || [ "${!1}" == "ON" ] || [ "${!1}" == "yes" ] || [ "${!1}" == "on" ]; then
         echo "YES"
-        return 1
+        return 0
     fi
-    if [ "${!1}" == "NO" ] || [ "${!1}" == "OFF" ]; then
+    if [ "${!1}" == "NO" ] || [ "${!1}" == "OFF" ] || [ "${!1}" == "no" ] || [ "${!1}" == "off" ]; then
         echo "NO"
-        return 1
+        return 0
     fi
-    echo "${txtred}Invalid value for $1 (${!1}) -- Must be ON, YES, NO, or OFF${txtrst}"
+    printf "${txtred}\nERROR: Invalid value for $1 (${!1}) -- Must be ON, YES, NO, or OFF\n\n${txtrst}" >& 2
     exit 1
 }
 
 #By default, download and then install.
 DOWNLOAD=${DOWNLOAD:-YES}
-DOWNLOAD=$(check_valid DOWNLOAD)
+DOWNLOAD=$(check_valid DOWNLOAD) || exit
 
 BUILD=${BUILD:-YES}
-BUILD=$(check_valid BUILD)
+BUILD=$(check_valid BUILD) || exit
 
 # Force downloading and installation even if the TPL already exists in lib/include
 FORCE=${FORCE:-NO}
-FORCE=$(check_valid FORCE)
+FORCE=$(check_valid FORCE) || exit
 
 DEBUG=${DEBUG:-NO}
-DEBUG=$(check_valid DEBUG)
+DEBUG=$(check_valid DEBUG) || exit
 
 # Shared libraries or static libraries?
 # For CRAY, must explicitly specify SHARED=YES, or it will default to NO
@@ -60,27 +60,36 @@ then
 else
     SHARED="${SHARED:-YES}"
 fi
-SHARED=$(check_valid SHARED)
+SHARED=$(check_valid SHARED) || exit
 
 # Enable Burst-Buffer support in PnetCDF?
 BB=${BB:-NO}
-BB=$(check_valid BB)
+BB=$(check_valid BB) || exit
 
 CRAY=${CRAY:-NO}
-CRAY=$(check_valid CRAY)
+CRAY=$(check_valid CRAY) || exit
 
-# Which TPLS? (HDF5 and NetCDF always, PnetCDF if MPI=ON)
-CGNS=${CGNS:-YES}
-CGNS=$(check_valid CGNS)
+MPI=${MPI:-NO}
+MPI=$(check_valid MPI) || exit
+
+# Which TPLS? (NetCDF always, but can be disabled if using an external version, PnetCDF if MPI=ON)
+NETCDF=${NETCDF:-YES}
+NETCDF=$(check_valid NETCDF) || exit
+PNETCDF=${PNETCDF:-${MPI}}
+PNETCDF=$(check_valid PNETCDF) || exit
+HDF5=${HDF5:-YES}
+HDF5=$(check_valid HDF5) || exit
+CGNS=${CGNS:-${HDF5}}
+CGNS=$(check_valid CGNS) || exit
 
 MATIO=${MATIO:-YES}
-MATIO=$(check_valid MATIO)
+MATIO=$(check_valid MATIO) || exit
 
 PARMETIS=${PARMETIS:-NO}
-PARMETIS=$(check_valid PARMETIS)
+PARMETIS=$(check_valid PARMETIS) || exit
 
 METIS=${METIS:-NO}
-METIS=$(check_valid METIS)
+METIS=$(check_valid METIS) || exit
 
 if [ "$PARMETIS" == "YES" ]
 then
@@ -88,13 +97,13 @@ then
 fi
 
 FMT=${FMT:-YES}
-FMT=$(check_valid FMT)
+FMT=$(check_valid FMT) || exit
 
 GNU_PARALLEL=${GNU_PARALLEL:-YES}
-GNU_PARALLEL=$(check_valid GNU_PARALLEL)
+GNU_PARALLEL=$(check_valid GNU_PARALLEL) || exit
 
 USE_ZLIB_NG=${USE_ZLIB_NG:-NO}
-USE_ZLIB_NG=$(check_valid USE_ZLIB_NG)
+USE_ZLIB_NG=$(check_valid USE_ZLIB_NG) || exit
 
 if [ "${USE_ZLIB_NG}" == "YES" ]
 then
@@ -102,40 +111,47 @@ then
 fi
 
 NEEDS_ZLIB=${NEEDS_ZLIB:-NO}
-NEEDS_ZLIB=$(check_valid NEEDS_ZLIB)
+NEEDS_ZLIB=$(check_valid NEEDS_ZLIB) || exit
 
 NEEDS_SZIP=${NEEDS_SZIP:-NO}
-NEEDS_SZIP=$(check_valid NEEDS_SZIP)
+NEEDS_SZIP=$(check_valid NEEDS_SZIP) || exit
 
 USE_AEC=${USE_AEC:-NO}
-USE_AEC=$(check_valid USE_AEC)
+USE_AEC=$(check_valid USE_AEC) || exit
 
 KOKKOS=${KOKKOS:-NO}
-KOKKOS=$(check_valid KOKKOS)
+KOKKOS=$(check_valid KOKKOS) || exit
 
 H5VERSION=${H5VERSION:-V114}
+# Build/Install the HDF5 C++ library?
+H5CPP=${H5CPP:-NO}
+H5CPP=$(check_valid H5CPP) || exit
 
 FAODEL=${FAODEL:-NO}
-FAODEL=$(check_valid FAODEL)
+FAODEL=$(check_valid FAODEL) || exit
+
+BOOST=${BOOST:-NO}
+BOOST=$(check_valid BOOST) || exit
 
 ADIOS2=${ADIOS2:-NO}
-ADIOS2=$(check_valid ADIOS2)
+ADIOS2=$(check_valid ADIOS2) || exit
 
 CATALYST2=${CATALYST2:-NO}
-CATALYST2=$(check_valid CATALYST2)
+CATALYST2=$(check_valid CATALYST2) || exit
 
-GTEST=${GTEST:-NO}
-GTEST=$(check_valid GTEST)
+GTEST=${GTEST:-${FAODEL}}
+GTEST=$(check_valid GTEST) || exit
+
+CATCH2=${CATCH2:-YES}
+CATCH2=$(check_valid CATCH2) || exit
 
-MPI=${MPI:-NO}
-MPI=$(check_valid MPI)
 
 SUDO=${SUDO:-}
 JOBS=${JOBS:-2}
 VERBOSE=${VERBOSE:-1}
 
 USE_PROXY=${USE_PROXY:-NO}
-USE_PROXY=$(check_valid USE_PROXY)
+USE_PROXY=$(check_valid USE_PROXY) || exit
 
 if [ "${USE_PROXY}" == "YES" ]
 then
@@ -143,7 +159,7 @@ then
     export https_proxy="https://proxy.sandia.gov:80"
 fi
 
-pwd
+echo "Current Location = $(pwd)"
 INSTALL_PATH=${INSTALL_PATH:-${ACCESS}}
 
 if [ "$MPI" == "YES" ] && [ "$CRAY" == "YES" ]
@@ -176,7 +192,7 @@ if [ $# -gt 0 ]; then
         echo "   ACCESS       = ${txtgrn}${ACCESS}${txtcyn} (Automatically set to current directory)"
         echo "   INSTALL_PATH = ${txtgrn}${INSTALL_PATH}${txtcyn}"
         echo "   OS           = ${txtgrn}${OS}${txtcyn} (Automatically set)"
-        echo "   COMPILER     = ${COMPILER}  (gnu clang intel ibm)"
+        echo "   COMPILER     = ${COMPILER}  (gnu clang intel ibm gnubrew gnumacport clangmacport nvidia)"
         echo "   MPI          = ${MPI} (Parallel Build?)"
         echo ""
         echo "   FORCE        = ${FORCE}"
@@ -186,7 +202,11 @@ if [ $# -gt 0 ]; then
         echo "   DEBUG        = ${DEBUG}"
         echo "   USE_PROXY    = ${USE_PROXY}"
         echo ""
+        echo "   NETCDF       = ${NETCDF}"
+        echo "   PNETCDF      = ${PNETCDF}"
+        echo "   HDF5         = ${HDF5}"
         echo "   H5VERSION    = ${H5VERSION}"
+        echo "   H5CPP        = ${H5CPP}"
         echo "   CGNS         = ${CGNS}"
         echo "   MATIO        = ${MATIO}"
         echo "   METIS        = ${METIS}"
@@ -202,6 +222,7 @@ if [ $# -gt 0 ]; then
         echo "   FAODEL       = ${FAODEL}"
         echo "   ADIOS2       = ${ADIOS2}"
         echo "   CATALYST2    = ${CATALYST2}"
+        echo "   CATCH2       = ${CATCH2}"
         echo "   GTEST        = ${GTEST}"
         echo ""
         echo "   SUDO         = ${SUDO} (empty unless need superuser permission via 'sudo')"
@@ -215,13 +236,13 @@ fi
 # Check that cmake, git, wget exist at the beginning instead of erroring out later on...
 check_exec cmake
 check_exec git
-check_exec wget
+check_exec curl
 
 if [ "$NEEDS_SZIP" == "YES" ]
 then
 if [ "$USE_AEC" == "YES" ]
 then
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libsz.${LD_EXT} ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_SZIP" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libsz.${LD_EXT} ]
     then
         echo "${txtgrn}+++ SZIP (via libaec library)${txtrst}"
         szip_version="1.0.4"
@@ -233,7 +254,7 @@ then
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf libaec-${szip_version}
             rm -rf v-${szip_version}.tar.gz
-            wget --no-check-certificate https://github.com/MathisRosenhauer/libaec/archive/v${szip_version}.tar.gz
+            curl -O -L --insecure https://github.com/MathisRosenhauer/libaec/archive/v${szip_version}.tar.gz
             tar -xzf v${szip_version}.tar.gz
             rm -rf v${szip_version}.tar.gz
         fi
@@ -263,7 +284,7 @@ then
         echo "${txtylw}+++ SZIP already installed.  Skipping download and installation.${txtrst}"
     fi
 else
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libsz.${LD_EXT} ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_SZIP" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libsz.${LD_EXT} ]
     then
         echo "${txtgrn}+++ SZIP${txtrst}"
         szip_version="2.1.1"
@@ -275,7 +296,7 @@ else
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf szip-${szip_version}
             rm -rf szip-${szip_version}.tar.gz
-            wget --no-check-certificate https://support.hdfgroup.org/ftp/lib-external/szip/2.1.1/src/szip-${szip_version}.tar.gz
+            curl -O -L --insecure https://support.hdfgroup.org/ftp/lib-external/szip/2.1.1/src/szip-${szip_version}.tar.gz
             tar -xzf szip-${szip_version}.tar.gz
             rm -rf szip-${szip_version}.tar.gz
         fi
@@ -310,11 +331,11 @@ fi
 
 if [ "$NEEDS_ZLIB" == "YES" ]
 then
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libz.${LD_EXT} ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_ZLIB" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libz.${LD_EXT} ]
     then
         if [ "$USE_ZLIB_NG" == "YES" ]
         then
-            echo "${txtgrn}+++ ZLIB-NG${txtrst}"
+            echo "${txtgrn}+++ ZLIB-NG ${zlib_ng_version}${txtrst}"
             zlib_ng_version="develop"
 
             cd $ACCESS || exit
@@ -323,14 +344,13 @@ then
             then
                 echo "${txtgrn}+++ Downloading...${txtrst}"
                 rm -rf zlib-ng
-                git clone https://github.com/zlib-ng/zlib-ng
+                git clone --depth 1 --branch ${zlib_ng_version} https://github.com/zlib-ng/zlib-ng
             fi
 
             if [ "$BUILD" == "YES" ]
             then
                 echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
                 cd zlib-ng || exit
-                git checkout ${zlib_ng_version}
                 rm -rf build
                 mkdir build
                 cmake -Bbuild -DCMAKE_INSTALL_PREFIX=${INSTALL_PATH} -DZLIB_COMPAT=YES .
@@ -358,7 +378,7 @@ then
                 echo "${txtgrn}+++ Downloading...${txtrst}"
                 rm -rf zlib-${zlib_version}
                 rm -rf zlib-${zlib_version}.tar.gz
-                wget --no-check-certificate https://zlib.net/fossils/zlib-${zlib_version}.tar.gz
+                curl -O -L --insecure https://zlib.net/fossils/zlib-${zlib_version}.tar.gz
                 tar -xzf zlib-${zlib_version}.tar.gz
                 rm -rf zlib-${zlib_version}.tar.gz
             fi
@@ -391,103 +411,101 @@ then
 fi
 
 # =================== BUILD HDF5 ===============
-if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libhdf5.${LD_EXT} ]
+if [ "$HDF5" == "YES" ]
 then
-    echo "${txtgrn}+++ HDF5${txtrst}"
-    if [ "${H5VERSION}" == "V18" ]; then
-        hdf_version="1.8.21"
-        hdf_base="1.8"
-    elif [ "${H5VERSION}" == "V110" ]; then
-        hdf_version="1.10.9"
-        hdf_base="1.10"
-    elif [ "${H5VERSION}" == "V112" ]; then
-        hdf_version="1.12.2"
-        hdf_base="1.12"
-    elif [ "${H5VERSION}" == "V113" ]; then
-        hdf_version="1.13.1"
-        hdf_base="1.13"
-    elif [ "${H5VERSION}" == "V114" ]; then
-        hdf_version="1.14.0"
-        hdf_base="1.14"
-    elif [ "${H5VERSION}" == "develop" ]; then
-        hdf_version="develop"
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_HDF5" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libhdf5.${LD_EXT} ]
+    then
+	hdf_suffix=""
+	if [ "${H5VERSION}" == "V110" ]; then
+	    hdf_version="hdf5-1_10_11"
+	elif [ "${H5VERSION}" == "V112" ]; then
+            hdf_version="hdf5-1_12_3"
+	elif [ "${H5VERSION}" == "V114" ]; then
+            hdf_version="hdf5_1.14.6"
+	elif [ "${H5VERSION}" == "develop" ]; then
+            hdf_version="develop"
+	else
+            echo 1>&2 ${txtred}Invalid HDF5 version specified: ${H5VERSION}.  Must be one of V110, V112, or V114 [default]. exiting.${txtrst}
+            exit 1
+	fi
+
+	echo "${txtgrn}+++ HDF5 ${hdf_version}${txtrst}"
+	
+	cd $ACCESS || exit
+	cd TPL/hdf5 || exit
+	if [ "$DOWNLOAD" == "YES" ]
+	then
+            echo "${txtgrn}+++ Downloading...${txtrst}"
+            rm -rf hdf5-${hdf_version}${hdf_suffix}
+            rm -f hdf5-${hdf_version}${hdf_suffix}.tar.bz2
+            if [ "${H5VERSION}" == "develop" ]; then
+		git clone --depth=1 https://github.com/HDFGroup/hdf5.git hdf5-develop
+            else
+		curl -O -L --insecure https://github.com/HDFGroup/hdf5/archive/refs/tags/${hdf_version}.tar.gz
+            fi
+            if [ "${H5VERSION}" != "develop" ]
+            then
+		tar -zxf ${hdf_version}.tar.gz
+		rm -f ${hdf_version}.tar.gz
+            fi
+	fi
+	
+	if [ "$BUILD" == "YES" ]
+	then
+            echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
+            cd hdf5-${hdf_version} || exit
+            rm -rf build
+            mkdir build
+            cd build || exit
+            CRAY=${CRAY} H5CPP=${H5CPP} H5VERSION=${H5VERSION} DEBUG=${DEBUG} SHARED=${SHARED} NEEDS_ZLIB=${NEEDS_ZLIB} NEEDS_SZIP=${NEEDS_SZIP} MPI=${MPI} bash -x ../../runcmake.sh
+            #CRAY=${CRAY} H5VERSION=${H5VERSION} DEBUG=${DEBUG} SHARED=${SHARED} NEEDS_ZLIB=${NEEDS_ZLIB} NEEDS_SZIP=${NEEDS_SZIP} MPI=${MPI} bash ../runconfigure.sh
+            if [[ $? != 0 ]]
+            then
+		echo 1>&2 ${txtred}couldn\'t configure hdf5. exiting.${txtrst}
+		exit 1
+            fi
+            make -j${JOBS} && ${SUDO} make "V=${VERBOSE}" install
+            if [[ $? != 0 ]]
+            then
+		echo 1>&2 ${txtred}couldn\'t build hdf5. exiting.${txtrst}
+		exit 1
+            fi
+	fi
+	# Create default plugin directory...
+	mkdir  ${INSTALL_PATH}/lib
+	mkdir  ${INSTALL_PATH}/lib/hdf5
+	mkdir  ${INSTALL_PATH}/lib/hdf5/lib
+	mkdir  ${INSTALL_PATH}/lib/hdf5/lib/plugin
     else
-        echo 1>&2 ${txtred}Invalid HDF5 version specified: ${H5VERSION}.  Must be one of V18, V110, V112. exiting.${txtrst}
-        exit 1
+	echo "${txtylw}+++ HDF5 already installed.  Skipping download and installation.${txtrst}"
     fi
-
-    cd $ACCESS || exit
-    cd TPL/hdf5 || exit
-    if [ "$DOWNLOAD" == "YES" ]
-    then
-        echo "${txtgrn}+++ Downloading...${txtrst}"
-        rm -rf hdf5-${hdf_version}
-        rm -f hdf5-${hdf_version}.tar.bz2
-        if [ "${H5VERSION}" == "develop" ]; then
-            git clone https://github.com/HDFGroup/hdf5.git hdf5-develop
-        else
-            wget --no-check-certificate https://support.hdfgroup.org/ftp/HDF5/releases/hdf5-${hdf_base}/hdf5-${hdf_version}/src/hdf5-${hdf_version}.tar.bz2
-        fi
-        if [ "${H5VERSION}" != "develop" ]
-        then
-            tar -jxf hdf5-${hdf_version}.tar.bz2
-            rm -f hdf5-${hdf_version}.tar.bz2
-        fi
-    fi
-
-    if [ "$BUILD" == "YES" ]
-    then
-        echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
-        cd hdf5-${hdf_version} || exit
-        rm -rf build
-        mkdir build
-        cd build || exit
-        CRAY=${CRAY} H5VERSION=${H5VERSION} DEBUG=${DEBUG} SHARED=${SHARED} NEEDS_ZLIB=${NEEDS_ZLIB} NEEDS_SZIP=${NEEDS_SZIP} MPI=${MPI} bash -x ../../runcmake.sh
-        #CRAY=${CRAY} H5VERSION=${H5VERSION} DEBUG=${DEBUG} SHARED=${SHARED} NEEDS_ZLIB=${NEEDS_ZLIB} NEEDS_SZIP=${NEEDS_SZIP} MPI=${MPI} bash ../runconfigure.sh
-        if [[ $? != 0 ]]
-        then
-            echo 1>&2 ${txtred}couldn\'t configure hdf5. exiting.${txtrst}
-            exit 1
-        fi
-        make -j${JOBS} && ${SUDO} make "V=${VERBOSE}" install
-        if [[ $? != 0 ]]
-        then
-            echo 1>&2 ${txtred}couldn\'t build hdf5. exiting.${txtrst}
-            exit 1
-        fi
-    fi
-    # Create default plugin directory...
-    mkdir  ${INSTALL_PATH}/lib/hdf5
-    mkdir  ${INSTALL_PATH}/lib/hdf5/lib
-    mkdir  ${INSTALL_PATH}/lib/hdf5/lib/plugin
-else
-    echo "${txtylw}+++ HDF5 already installed.  Skipping download and installation.${txtrst}"
 fi
+
 # =================== INSTALL PnetCDF if parallel build ===============
-if [ "$MPI" == "YES" ]
+if [ "$PNETCDF" == "YES" ] && [ "$MPI" == "YES" ]
 then
     # PnetCDF currently only builds static library...
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libpnetcdf.a ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_PNETCDF" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libpnetcdf.a ]
     then
         echo "${txtgrn}+++ PnetCDF${txtrst}"
-        pnet_version="1.12.2"
+        pnetcdf_version="1.14.0"
         pnet_base="pnetcdf"
         cd $ACCESS || exit
         cd TPL/pnetcdf || exit
         if [ "$DOWNLOAD" == "YES" ]
         then
             echo "${txtgrn}+++ Downloading...${txtrst}"
-            rm -rf ${pnet_base}-${pnet_version}
-            rm -f ${pnet_base}-${pnet_version}.tar.gz
-            wget --no-check-certificate https://parallel-netcdf.github.io/Release/${pnet_base}-${pnet_version}.tar.gz
-            tar -xzf ${pnet_base}-${pnet_version}.tar.gz
-            rm -f ${pnet_base}-${pnet_version}.tar.gz
+            rm -rf ${pnet_base}-${pnetcdf_version}
+            rm -f ${pnet_base}-${pnetcdf_version}.tar.gz
+            curl -O -L --insecure https://parallel-netcdf.github.io/Release/${pnet_base}-${pnetcdf_version}.tar.gz
+            tar -xzf ${pnet_base}-${pnetcdf_version}.tar.gz
+            rm -f ${pnet_base}-${pnetcdf_version}.tar.gz
         fi
 
         if [ "$BUILD" == "YES" ]
         then
             echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
-            cd ${pnet_base}-${pnet_version} || exit
+            cd ${pnet_base}-${pnetcdf_version} || exit
             CRAY=${CRAY} BB=${BB} DEBUG=${DEBUG} SHARED=${SHARED} bash ../runconfigure.sh
             if [[ $? != 0 ]]
             then
@@ -514,35 +532,43 @@ then
 fi
 
 # =================== INSTALL NETCDF ===============
-if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libnetcdf.${LD_EXT} ]
+if [ "$NETCDF" == "YES" ]
 then
-    echo "${txtgrn}+++ NetCDF${txtrst}"
+if [ "$FORCE" == "YES" ] || [ "$FORCE_NETCDF" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libnetcdf.${LD_EXT} ]
+then
+#   netcdf_version="v4.9.1"
+#    netcdf_version="v4.9.2"
+    netcdf_version="v4.9.3"
+#   netcdf_version="v4.8.1"
+#   netcdf_version="main"
+
+    echo "${txtgrn}+++ NetCDF ${netcdf_version} ${txtrst}"
     cd $ACCESS || exit
     cd TPL/netcdf || exit
     if [ "$DOWNLOAD" == "YES" ]
     then
         echo "${txtgrn}+++ Downloading...${txtrst}"
         rm -rf netcdf-c
-        git clone https://github.com/Unidata/netcdf-c netcdf-c
+        git clone --depth 1 --branch ${netcdf_version} https://github.com/Unidata/netcdf-c netcdf-c
     fi
 
-   net_version="v4.9.0"
-#   net_version="v4.8.1"
-#   net_version="master"
+    if [ "$netcdf_version" == "v4.9.3" ]
+    then
+	PREFIX="NETCDF_"
+    fi
 
     if [ "$BUILD" == "YES" ]
     then
         echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
         cd netcdf-c || exit
-        if [ "$net_version" != "master" ]
-           then
-               git checkout $net_version
-        fi
         rm -rf build
         mkdir build
         cd build || exit
-        export HDF5_PLUGIN_PATH=${INSTALL_PATH}/lib/hdf5/lib/plugin
-        CRAY=${CRAY} SHARED=${SHARED} DEBUG=${DEBUG} NEEDS_ZLIB=${NEEDS_ZLIB} MPI=${MPI} bash -x ../../runcmake.sh
+	if [ "$HDF5" == "YES" ]
+	then
+           export HDF5_PLUGIN_PATH=${INSTALL_PATH}/lib/hdf5/lib/plugin
+	fi
+        PREFIX=${PREFIX} CRAY=${CRAY} SHARED=${SHARED} DEBUG=${DEBUG} HDF5=${HDF5} NEEDS_ZLIB=${NEEDS_ZLIB} MPI=${MPI} bash -x ../../runcmake.sh
         if [[ $? != 0 ]]
         then
             echo 1>&2 ${txtred}couldn\'t configure cmake for NetCDF. exiting.${txtrst}
@@ -559,26 +585,37 @@ then
 else
     echo "${txtylw}+++ NetCDF already installed.  Skipping download and installation.${txtrst}"
 fi
+fi
 # =================== INSTALL CGNS ===============
-if [ "$CGNS" == "YES" ]
+if [ "$CGNS" == "YES" ] && [ "$HDF5" == "YES" ]
 then
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libcgns.${LD_EXT} ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_CGNS" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libcgns.${LD_EXT} ]
     then
-        echo "${txtgrn}+++ CGNS${txtrst}"
+	cgns_version="v4.5.0"
+        echo "${txtgrn}+++ CGNS ${cgns_version} ${txtrst}"
         cd $ACCESS || exit
         cd TPL/cgns || exit
         if [ "$DOWNLOAD" == "YES" ]
         then
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf CGNS
-            git clone https://github.com/cgns/CGNS
+            git clone --depth 1 --branch ${cgns_version} https://github.com/cgns/CGNS
         fi
 
         if [ "$BUILD" == "YES" ]
         then
             echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
             cd CGNS || exit
-            git checkout v4.3.0
+            patch -p1 < ../CGNS-Allow-more-liberal-version-matching.patch
+            if [[ $? != 0 ]]
+            then
+                echo 1>&2 ${txtred}Problems applying CGNS Version patch. Continuing, but look here if errors building. Files written may not be readable by older libraries.${txtrst}
+            fi
+            patch < ../CGNS-ZLIB.patch
+            if [[ $? != 0 ]]
+            then
+                echo 1>&2 ${txtred}Problems applying CGNS ZLIB patch.${txtrst}
+            fi
             rm -rf build
             mkdir build
             cd build || exit
@@ -600,11 +637,15 @@ then
         echo "${txtylw}+++ CGNS already installed.  Skipping download and installation.${txtrst}"
     fi
 fi
+if [ "$CGNS" == "YES" ] && [ "$HDF5" == "NO" ]
+then
+    echo "${txtred}+++ CGNS requested, but HDF5 is not enabled.  Cannot build CGNS without HDF5.${txtrst}"
+fi
 
 # =================== INSTALL METIS  ===============
 if [ "$METIS" == "YES" ]
 then
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libmetis.a ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_METIS" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libmetis.a ]
     then
         echo "${txtgrn}+++ Metis${txtrst}"
         cd $ACCESS || exit
@@ -614,7 +655,7 @@ then
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf metis-5.1.0
             rm -f metis-5.1.0.tar.gz
-            wget --no-check-certificate https://github.com/scivision/METIS/archive/v5.1.0.1.tar.gz
+            curl -O -L --insecure https://github.com/scivision/METIS/archive/v5.1.0.1.tar.gz
             tar zxvf v5.1.0.1.tar.gz
         fi
 
@@ -646,7 +687,7 @@ fi
 # =================== INSTALL PARMETIS  ===============
 if [ "$PARMETIS" == "YES" ] && [ "$MPI" == "YES" ]
 then
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libparmetis.a ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_PARMETIS" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libparmetis.a ]
     then
         echo "${txtgrn}+++ ParMETIS${txtrst}"
         cd $ACCESS || exit
@@ -655,7 +696,7 @@ then
         then
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf parmetis
-            git clone https://github.com/gsjaardema/parmetis
+            git clone --depth=1 https://github.com/gsjaardema/parmetis
         fi
 
         if [ "$BUILD" == "YES" ]
@@ -689,28 +730,27 @@ then
     check_exec automake
     check_exec autoconf
 
-    matio_version="1.5.23"
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libmatio.${LD_EXT} ]
+    matio_version="v1.5.28"
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_MATIO" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libmatio.${LD_EXT} ]
     then
-        echo "${txtgrn}+++ MatIO${txtrst}"
+        echo "${txtgrn}+++ MatIO ${matio_version} ${txtrst}"
         cd $ACCESS || exit
         cd TPL/matio || exit
         if [ "$DOWNLOAD" == "YES" ]
         then
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf matio
-            git clone https://github.com/tbeu/matio.git
+            git clone --depth 1 --branch ${matio_version} https://github.com/tbeu/matio.git
         fi
 
         if [ "$BUILD" == "YES" ]
         then
             echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
             cd matio || exit
-            git checkout ${matio_version}
             rm -rf build
             mkdir build
             cd build || exit
-            CRAY=${CRAY} SHARED=${SHARED} DEBUG=${DEBUG} NEEDS_ZLIB=${NEEDS_ZLIB} bash -x ../../runcmake.sh
+            CRAY=${CRAY} SHARED=${SHARED} DEBUG=${DEBUG} NEEDS_ZLIB=${NEEDS_ZLIB} HDF5=${HDF5} bash -x ../../runcmake.sh
             if [[ $? != 0 ]]
             then
                 echo 1>&2 ${txtred}couldn\'t configure MatIO. exiting.${txtrst}
@@ -731,25 +771,24 @@ fi
 # =================== INSTALL FMT ===============
 if [ "$FMT" == "YES" ]
 then
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/include/fmt/core.h ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_FMT" == "YES" ] || ! [ -e $INSTALL_PATH/include/fmt/core.h ]
     then
-        echo "${txtgrn}+++ FMT${txtrst}"
+        fmt_version="11.1.4"
+        echo "${txtgrn}+++ FMT ${fmt_version} ${txtrst}"
         cd $ACCESS || exit
         cd TPL/fmt || exit
-        fmt_version="9.1.0"
 
         if [ "$DOWNLOAD" == "YES" ]
         then
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf fmt
-            git clone https://github.com/fmtlib/fmt
+            git clone --depth 1 --branch ${fmt_version} https://github.com/fmtlib/fmt
         fi
 
         if [ "$BUILD" == "YES" ]
         then
             echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
             cd fmt || exit
-            git checkout ${fmt_version}
             rm -rf build
             mkdir build
             cd build || exit
@@ -775,9 +814,9 @@ fi
 # =================== INSTALL KOKKOS  ===============
 if [ "$KOKKOS" == "YES" ]
 then
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libkokkoscore.${LD_EXT} ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_KOKKOS" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libkokkoscore.${LD_EXT} ]
     then
-        kokkos_version="3.6.00"
+        kokkos_version="4.5.01"
         echo "${txtgrn}+++ KOKKOS${txtrst}"
         cd $ACCESS || exit
         cd TPL/kokkos || exit
@@ -785,9 +824,9 @@ then
         then
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf kokkos
-            wget --no-check-certificate https://github.com/kokkos/kokkos/archive/${kokkos_version}.tar.gz
-            tar -zxf ${kokkos_version}.tar.gz
-            rm -f ${kokkos_version}.tar.gz
+            curl -O -L --insecure https://github.com/kokkos/kokkos/releases/download/${kokkos_version}/kokkos-${kokkos_version}.tar.gz
+            tar -zxf kokkos-${kokkos_version}.tar.gz
+            rm -f kokkos-${kokkos_version}.tar.gz
             ln -s kokkos-${kokkos_version} kokkos
         fi
 
@@ -820,23 +859,23 @@ fi
 # =================== INSTALL ADIOS2  ===============
 if [ "$ADIOS2" == "YES" ]
 then
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libadios2.${LD_EXT} ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_ADIOS2" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libadios2_c.${LD_EXT} ]
     then
-        echo "${txtgrn}+++ ADIOS2${txtrst}"
+	adios2_version="v2.10.2"
+        echo "${txtgrn}+++ ADIOS2 ${adios2_version} ${txtrst}"
         cd $ACCESS || exit
         cd TPL/adios2 || exit
         if [ "$DOWNLOAD" == "YES" ]
         then
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf ADIOS2
-            git clone https://github.com/ornladios/ADIOS2.git
+	    git clone --depth 1 --branch ${adios2_version} https://github.com/ornladios/ADIOS2.git
         fi
 
         if [ "$BUILD" == "YES" ]
         then
             echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
             cd ADIOS2 || exit
-            git checkout v2.5.0
             rm -rf build
             mkdir build
             cd build || exit
@@ -862,23 +901,23 @@ fi
 # =================== INSTALL CATALYST2  ===============
 if [ "$CATALYST2" == "YES" ]
 then
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libcatalyst.${LD_EXT} ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_CATALYST2" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libcatalyst.${LD_EXT} ]
     then
-        echo "${txtgrn}+++ Catalyst2${txtrst}"
+        catalyst2_version="v2.0.0"
+        echo "${txtgrn}+++ Catalyst2 ${catalyst2_version} ${txtrst}"
         cd $ACCESS || exit
         cd TPL/catalyst2 || exit
         if [ "$DOWNLOAD" == "YES" ]
         then
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf catalyst
-            git clone https://gitlab.kitware.com/paraview/catalyst.git
+            git clone --depth 1 --branch ${catalyst2_version} https://gitlab.kitware.com/paraview/catalyst.git
         fi
 
         if [ "$BUILD" == "YES" ]
         then
             echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
             cd catalyst || exit
-                  git checkout master #todo: a specific version
             rm -rf build
             mkdir build
             cd build || exit
@@ -904,23 +943,23 @@ fi
 # =================== INSTALL gtest  ===============
 if [ "$GTEST" == "YES" ]
 then
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libgtest.${LD_EXT} ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_GTEST" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libgtest.${LD_EXT} ]
     then
-        echo "${txtgrn}+++ gtest${txtrst}"
+        gtest_version="release-1.11.0"
+        echo "${txtgrn}+++ gtest ${gtest_version} ${txtrst}"
         cd $ACCESS || exit
         cd TPL/gtest || exit
         if [ "$DOWNLOAD" == "YES" ]
         then
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf googletest
-            git clone https://github.com/google/googletest.git
+            git clone --branch ${gtest_version} --depth 1 https://github.com/google/googletest.git
         fi
 
         if [ "$BUILD" == "YES" ]
         then
             echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
             cd googletest || exit
-            git checkout release-1.11.0
             rm -rf build
             mkdir build
             cd build || exit
@@ -943,10 +982,52 @@ then
     fi
 fi
 
+# =================== INSTALL catch2  ===============
+if [ "$CATCH2" == "YES" ]
+then
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_CATCH2" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libCatch2.a ]
+    then
+        catch2_version="v3.8.1"
+        echo "${txtgrn}+++ Catch2 ${catch2_version} ${txtrst}"
+        cd $ACCESS || exit
+        cd TPL/catch2 || exit
+        if [ "$DOWNLOAD" == "YES" ]
+        then
+            echo "${txtgrn}+++ Downloading...${txtrst}"
+            rm -rf Catch2
+            git clone --branch ${catch2_version} --depth 1 https://github.com/catchorg/Catch2.git
+        fi
+
+        if [ "$BUILD" == "YES" ]
+        then
+            echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
+            cd Catch2 || exit
+            rm -rf build
+            mkdir build
+            cd build || exit
+            CRAY=${CRAY} SHARED=${SHARED} DEBUG=${DEBUG} bash -x ../../runcmake.sh
+            if [[ $? != 0 ]]
+            then
+                echo 1>&2 ${txtred}couldn\'t configure cmake for Catch2. exiting.${txtrst}
+                exit 1
+            fi
+
+            make -j${JOBS} && ${SUDO} make "VERBOSE=${VERBOSE}" install
+            if [[ $? != 0 ]]
+            then
+                echo 1>&2 ${txtred}couldn\'t build Catch2. exiting.${txtrst}
+                exit 1
+            fi
+        fi
+    else
+        echo "${txtylw}+++ Catch2 already installed.  Skipping download and installation.${txtrst}"
+    fi
+fi
+
 # =================== INSTALL PARALLEL  ===============
 if [ "$GNU_PARALLEL" == "YES" ]
 then
-    if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/bin/env_parallel ]
+    if [ "$FORCE" == "YES" ] || [ "$FORCE_GNU_PARALLEL" == "YES" ] || ! [ -e $INSTALL_PATH/bin/env_parallel ]
     then
         echo "${txtgrn}+++ GNU Parallel${txtrst}"
         cd $ACCESS || exit
@@ -955,7 +1036,7 @@ then
         then
             echo "${txtgrn}+++ Downloading...${txtrst}"
             rm -rf parallel-*
-            wget --no-check-certificate https://ftp.gnu.org/gnu/parallel/parallel-latest.tar.bz2
+            curl -O -L --insecure https://ftp.gnu.org/gnu/parallel/parallel-latest.tar.bz2
             tar -jxf parallel-latest.tar.bz2
             rm -rf parallel-latest.tar.bz2
         fi
@@ -982,10 +1063,37 @@ then
     fi
 fi
 
+# =================== INSTALL BOOST ===============
+if [ "$BOOST" == "YES" ]
+then
+  if [ "$FORCE" == "YES" ] || [ "$FORCE_BOOST" == "YES" ] || ! [ -e $INSTALL_PATH/include/boost ]
+  then
+    # FAODEL Requires Boost... For now, just download and install
+    echo "${txtgrn}+++ Installing Boost as dependency of Faodel${txtrst}"
+    cd $ACCESS || exit
+    cd TPL/boost || exit
+    BOOST_VER="1_82_0"
+    if [ "$DOWNLOAD" == "YES" ]
+    then
+	curl -O -L --insecure "https://boostorg.jfrog.io/artifactory/main/release/1.82.0/source/boost_${BOOST_VER}.tar.bz2" 
+	tar xf boost_${BOOST_VER}.tar.bz2
+    fi
+    if [ "$BUILD" == "YES" ]
+    then
+	echo "${txtgrn}+++ Configuring, Building, and Installing...${txtrst}"
+	cd boost_${BOOST_VER}
+	./bootstrap.sh --prefix=${INSTALL_PATH}
+	./b2 -a install
+    fi
+  else
+    echo "${txtylw}+++ Boost already installed.  Skipping download and installation.${txtrst}"
+  fi
+fi
+
 # =================== INSTALL FAODEL ===============
 if [ "$FAODEL" == "YES" ]
 then
-  if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libkelpie.a ]
+  if [ "$FORCE" == "YES" ] || [ "$FORCE_FAODEL" == "YES" ] || ! [ -e $INSTALL_PATH/lib/libkelpie.${LD_EXT} ]
   then
     faodel_base="faodel"
     echo "${txtgrn}+++ Faodel${txtrst}"
@@ -995,7 +1103,7 @@ then
     then
       echo "${txtgrn}+++ Downloading...${txtrst}"
       rm -rf faodel*
-      git clone git@github.com:faodel/faodel.git
+      git clone --depth=1 https://github.com/sandialabs/faodel.git
     fi
 
     if [ "$BUILD" == "YES" ]
@@ -1027,7 +1135,7 @@ fi
 if [ "$FAODEL" == "YES" ]
 then
   # Currently, the FAODEL backend requires cereal, so if Faodel is enabled, we'll install cereal, too.
-  if [ "$FORCE" == "YES" ] || ! [ -e $INSTALL_PATH/include/cereal/archives/portable_binary.hpp ]
+  if [ "$FORCE" == "YES" ] || [ "$FORCE_FAODEL" == "YES" ] || ! [ -e $INSTALL_PATH/include/cereal/archives/portable_binary.hpp ]
   then
     echo "${txtgrn}+++ Cereal${txtrst}"
     cd $ACCESS || exit
@@ -1040,7 +1148,7 @@ then
     then
       echo "${txtgrn}+++ Downloading...${txtrst}"
       rm -rf cereal*
-      wget --no-check-certificate https://github.com/USCiLab/cereal/archive/v1.3.0.tar.gz
+      curl -O -L --insecure https://github.com/USCiLab/cereal/archive/v1.3.0.tar.gz
       tar xzf v1.3.0.tar.gz
       rm -f v1.3.0.tar.gz
       cp -R cereal-1.3.0/include/cereal $INSTALL_PATH/include/
