@@ -1,14 +1,16 @@
 /*
- * Copyright(C) 1999-2023 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2025 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
  * See packages/seacas/LICENSE for details
  */
 #pragma once
+#include <array>
+#include <string>
 
-static char const *qainfo[] = {
+static const std::array<std::string, 3> qainfo{
     "epu --  E Pluribus Unum",
-    "6.06",
-    "2023/01/11",
+    "7.01",
+    "2024/03/13",
 };
