@@ -1,23 +1,24 @@
-// Copyright(C) 1999-2022 National Technology & Engineering Solutions
+// Copyright(C) 1999-2025 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
 // See packages/seacas/LICENSE for details
 
 #include "apr_scanner.h" // for Scanner
-#include "apr_symrec.h"
 #include "apr_util.h"
 #include "aprepro.h"        // for Aprepro, symrec, file_rec, etc
 #include "aprepro_parser.h" // for Parser, Parser::token, etc
-#include "terminal_color.h"
-#include <cstdlib>  // for exit, EXIT_SUCCESS, etc
-#include <cstring>  // for strcmp
+#include <cstdio>
+#include <cstdlib> // for exit, EXIT_SUCCESS, etc
+#include <cstring> // for strcmp
+#include <fmt/color.h>
+#include <fmt/format.h>
+#include <fmt/ostream.h>
 #include <fstream>  // for operator<<, basic_ostream, etc
 #include <iomanip>  // for operator<<, setw, etc
 #include <iostream> // for left, cerr, cout, streampos
 #include <stack>    // for stack
 #include <stdexcept>
-#include <stdio.h>
 #include <string> // for string, operator==, etc
 #include <unistd.h>
 #include <vector> // for allocator, vector
@@ -32,7 +33,9 @@
 #endif
 
 namespace {
-  const char *version_string = "6.11 (2022/10/18)";
+  const std::string version_short{"6.39"};
+  const std::string version_date{"(2025/05/21)"};
+  const std::string version_string = version_short + " " + version_date;
 
   void output_copyright();
 
@@ -49,6 +52,16 @@ namespace {
     }
     return value;
   }
+
+  void format_option(std::string_view option, std::string_view short_opt,
+                     std::string_view description)
+  {
+    fmt::print(stderr, fmt::emphasis::bold, "{}", option);
+    if (!short_opt.empty()) {
+      fmt::print(stderr, " or {}", fmt::styled(short_opt, fmt::emphasis::bold));
+    }
+    fmt::print(stderr, ":  {}\n", description);
+  }
 } // namespace
 
 namespace SEAMS {
@@ -57,8 +70,8 @@ namespace SEAMS {
     Symtable() = default;
     ~Symtable()
     {
-      for (auto &sym : sym_table) {
-        auto &ptr = sym.second;
+      for (const auto &sym : sym_table) {
+        const auto &ptr = sym.second;
         delete ptr;
       }
     }
@@ -88,20 +101,30 @@ namespace SEAMS {
 
   Aprepro::Aprepro() : sym_table(new Symtable())
   {
-    ap_file_list.push(file_rec());
+    ap_file_list.emplace(file_rec());
     init_table("$");
     aprepro = this;
+
+    add_variable("__loop_level__", 0, false, true);
   }
 
   Aprepro::~Aprepro()
   {
     if (!outputStream.empty()) {
       outputStream.top()->flush();
+      while (outputStream.size() > 1) {
+        std::ostream *output = outputStream.top();
+        outputStream.pop();
+        delete output;
+      }
     }
 
     // May need to delete this if set via --info=filename command.
     // May need a flag to determine this...
     infoStream->flush();
+    if (closeInfo) {
+      delete infoStream;
+    }
 
     if ((stringScanner != nullptr) && stringScanner != lexer) {
       delete stringScanner;
@@ -119,11 +142,12 @@ namespace SEAMS {
     cleanup_memory();
   }
 
-  std::string Aprepro::version() { return version_string; }
+  const std::string &Aprepro::version() { return version_string; }
+  const std::string &Aprepro::short_version() { return version_short; }
 
   std::string Aprepro::long_version() const
   {
-    auto comment = getsym("_C_")->value.svar;
+    const auto &comment = getsym("_C_")->value.svar;
     return comment + " Algebraic Preprocessor (Aprepro) version " + version();
   }
 
@@ -170,7 +194,7 @@ namespace SEAMS {
   bool Aprepro::parse_strings(const std::vector<std::string> &input, const std::string &sname)
   {
     std::stringstream iss;
-    for (auto &elem : input) {
+    for (const auto &elem : input) {
       iss << elem << '\n';
     }
     return parse_stream(iss, sname);
@@ -208,27 +232,19 @@ namespace SEAMS {
 
   void Aprepro::error(const std::string &msg, bool line_info, bool prefix) const
   {
-    bool              colorize = (errorStream == &std::cerr) && isatty(fileno(stderr));
-    std::stringstream ss;
-    if (prefix) {
-      if (colorize) {
-        (*errorStream) << trmclr::red;
-      }
-      (*errorStream) << "Aprepro: ERROR: ";
-    }
+    bool        colorize = (errorStream == &std::cerr) && isatty(fileno(stderr));
+    std::string message  = prefix ? fmt::format("Aprepro-{}: ERROR: {}", short_version(), msg)
+                                  : fmt::format("{}", msg);
+    std::string lines    = line_info ? fmt::format(" ({}, line {})", ap_file_list.top().name,
+                                                   ap_file_list.top().lineno)
+                                     : "";
 
-    ss << msg;
-    if (prefix && colorize) {
-      ss << trmclr::normal;
+    if (colorize) {
+      fmt::print(*errorStream, "{}{}\n", fmt::styled(message, fmt::fg(fmt::color::red)), lines);
     }
-
-    if (line_info) {
-      ss << " (" << ap_file_list.top().name << ", line " << ap_file_list.top().lineno << ")";
+    else {
+      fmt::print(*errorStream, "{}{}\n", message, lines);
     }
-    ss << "\n";
-
-    // Send it to the user defined stream
-    (*errorStream) << ss.str();
     parseErrorCount++;
   }
 
@@ -238,28 +254,20 @@ namespace SEAMS {
       return;
     }
 
-    bool              colorize = (warningStream == &std::cerr) && isatty(fileno(stderr));
-    std::stringstream ss;
-    if (prefix) {
-      if (colorize) {
-        (*warningStream) << trmclr::yellow;
-      }
-      (*warningStream) << "Aprepro: WARNING: ";
+    bool        colorize = (warningStream == &std::cerr) && isatty(fileno(stderr));
+    std::string message =
+        prefix ? fmt::format("Aprepro: WARNING: {}", msg) : fmt::format("{}", msg);
+    std::string lines = line_info ? fmt::format(" ({}, line {})", ap_file_list.top().name,
+                                                ap_file_list.top().lineno)
+                                  : "";
+
+    if (colorize) {
+      fmt::print(*warningStream, "{}{}\n", fmt::styled(message, fmt::fg(fmt::color::yellow)),
+                 lines);
     }
-
-    ss << msg;
-
-    if (prefix && colorize) {
-      ss << trmclr::normal;
+    else {
+      fmt::print(*warningStream, "{}{}\n", message, lines);
     }
-
-    if (line_info) {
-      ss << " (" << ap_file_list.top().name << ", line " << ap_file_list.top().lineno << ")";
-    }
-    ss << "\n";
-
-    // Send it to the user defined stream
-    (*warningStream) << ss.str();
     parseWarningCount++;
   }
 
@@ -269,41 +277,42 @@ namespace SEAMS {
       return;
     }
 
-    bool              colorize = (infoStream == &std::cout) && isatty(fileno(stdout));
-    std::stringstream ss;
-    if (prefix) {
-      if (colorize) {
-        (*infoStream) << trmclr::blue;
-      }
-      (*infoStream) << "Aprepro: INFO: ";
-    }
-    ss << msg;
+    bool        colorize = (infoStream == &std::cout) && isatty(fileno(stdout));
+    std::string message  = prefix ? fmt::format("Aprepro: INFO: {}", msg) : fmt::format("{}", msg);
+    std::string lines    = line_info ? fmt::format(" ({}, line {})", ap_file_list.top().name,
+                                                   ap_file_list.top().lineno)
+                                     : "";
 
-    if (prefix && colorize) {
-      ss << trmclr::normal;
+    if (colorize) {
+      fmt::print(*infoStream, "{}{}\n", fmt::styled(message, fmt::fg(fmt::color::blue)), lines);
     }
-
-    if (line_info) {
-      ss << " (" << ap_file_list.top().name << ", line " << ap_file_list.top().lineno << ")";
+    else {
+      fmt::print(*infoStream, "{}{}\n", message, lines);
     }
-    ss << "\n";
+  }
 
-    // Send it to the user defined stream
-    (*infoStream) << ss.str();
+  void Aprepro::set_error_streams(std::ostream *c_error, std::ostream *c_warning,
+                                  std::ostream *c_info, bool close_error, bool close_warning,
+                                  bool close_info)
+  {
+    if (c_error != nullptr) {
+      errorStream = c_error;
+      closeError  = close_error;
+    }
+    if (c_warning != nullptr) {
+      warningStream = c_warning;
+      closeWarning  = close_warning;
+    }
+    if (c_info != nullptr) {
+      infoStream = c_info;
+      closeInfo  = close_info;
+    }
   }
 
   void Aprepro::set_error_streams(std::ostream *c_error, std::ostream *c_warning,
                                   std::ostream *c_info)
   {
-    if (c_error != nullptr) {
-      errorStream = c_error;
-    }
-    if (c_warning != nullptr) {
-      warningStream = c_warning;
-    }
-    if (c_info != nullptr) {
-      infoStream = c_info;
-    }
+    set_error_streams(c_error, c_warning, c_info, false, false, false);
   }
 
   /* Two methods for opening files:
@@ -323,8 +332,7 @@ namespace SEAMS {
 
     /* See if file exists in current directory (or as specified) */
     auto pointer = new std::fstream(file, smode);
-    if ((pointer->bad() || !pointer->good()) &&
-        !ap_options.include_path.empty()) {
+    if ((pointer->bad() || !pointer->good()) && !ap_options.include_path.empty()) {
       /* If there is an include path specified, try opening file there */
       std::string file_path(ap_options.include_path);
       file_path += "/";
@@ -356,8 +364,7 @@ namespace SEAMS {
 
     auto pointer = new std::fstream(file, smode);
 
-    if ((pointer->bad() || !pointer->good()) &&
-        !ap_options.include_path.empty()) {
+    if ((pointer->bad() || !pointer->good()) && !ap_options.include_path.empty()) {
       /* If there is an include path specified, try opening file there */
       std::string file_path(ap_options.include_path);
       file_path += "/";
@@ -368,6 +375,22 @@ namespace SEAMS {
     return pointer;
   }
 
+  Aprepro::SYMBOL_TYPE Aprepro::get_symbol_type(const SEAMS::symrec *symbol) const
+  {
+    switch (symbol->type) {
+    case Parser::token::VAR: return SYMBOL_TYPE::VARIABLE;
+    case Parser::token::SVAR: return SYMBOL_TYPE::STRING_VARIABLE;
+    case Parser::token::AVAR: return SYMBOL_TYPE::ARRAY_VARIABLE;
+    case Parser::token::IMMVAR: return SYMBOL_TYPE::IMMUTABLE_VARIABLE;
+    case Parser::token::IMMSVAR: return SYMBOL_TYPE::IMMUTABLE_STRING_VARIABLE;
+    case Parser::token::UNDVAR: return SYMBOL_TYPE::UNDEFINED_VARIABLE;
+    case Parser::token::FNCT: return SYMBOL_TYPE::FUNCTION;
+    case Parser::token::SFNCT: return SYMBOL_TYPE::STRING_FUNCTION;
+    case Parser::token::AFNCT: return SYMBOL_TYPE::ARRAY_FUNCTION;
+    default: return SYMBOL_TYPE::INTERNAL;
+    }
+  }
+
   symrec *Aprepro::putsym(const std::string &sym_name, SYMBOL_TYPE sym_type, bool is_internal)
   {
     int  parser_type = 0;
@@ -391,13 +414,14 @@ namespace SEAMS {
       parser_type = Parser::token::AFNCT;
       is_function = true;
       break;
+    case SYMBOL_TYPE::INTERNAL: parser_type = Parser::token::UNDVAR; break;
     }
 
     // If the type is a function type, it can be overloaded as long as
     // it returns the same type which means that the "parser_type" is
     // the same.  If we have a function, see if it has already been
     // defined and if so, check that the parser_type matches and then
-    // retrn that pointer instead of creating a new symrec.
+    // return that pointer instead of creating a new symrec.
 
     if (is_function) {
       symrec *ptr = getsym(sym_name);
@@ -428,6 +452,33 @@ namespace SEAMS {
     return ptr;
   }
 
+  namespace {
+    bool match_option(const std::string &option, const std::string &long_opt,
+                      const std::string &short_opt, size_t min_length)
+    {
+      // See if `option` starts with 1 or 2 leading `-`.
+      size_t number_dash = option[0] == '-' ? (option[1] == '-' ? 2 : 1) : 0;
+
+      // See if `option` contains a `=`, save position...
+      auto equals = option.find('=');
+      if (equals == std::string::npos) {
+        equals = option.size();
+      }
+      else {
+        equals = equals - number_dash;
+      }
+
+      // NOTE: `option` contains two leading `--` or `-` and a single character...
+      if (!short_opt.empty() && number_dash == 1 && option.substr(1, equals) == short_opt) {
+        return true;
+      }
+
+      // Now deal with long options...
+      auto len_option = std::max(equals - 2, min_length);
+      return option.substr(2, len_option) == long_opt.substr(0, len_option);
+    }
+  } // namespace
+
   int Aprepro::set_option(const std::string &option, const std::string &optional_value)
   {
     // Option should be of the form "--option" or "-O"
@@ -440,65 +491,70 @@ namespace SEAMS {
     // Some options (--include)
     int ret_value = 0;
 
-    if (option == "--debug" || option == "-d") {
+    if (match_option(option, "debug", "d", 3)) {
       ap_options.debugging = true;
     }
-    if (option == "--dumpvars" || option == "-D") {
-      ap_options.dumpvars = true;
-    }
-    else if (option == "--dumpvars_json" || option == "-J") {
+    else if (match_option(option, "dumpvars_json", "J", 13)) {
       ap_options.dumpvars_json = true;
     }
-    else if (option == "--version" || option == "-v") {
+    else if (match_option(option, "dumpvars", "D", 4)) {
+      ap_options.dumpvars = true;
+    }
+    else if (match_option(option, "version", "v", 3)) {
       std::cerr << "Algebraic Preprocessor (Aprepro) version " << version() << "\n";
       exit(EXIT_SUCCESS);
     }
-    else if (option == "--nowarning" || option == "-W") {
+    else if (match_option(option, "quiet", "q", 5)) {
+      // Do nothing, but don't report an error/warning.  Handled elsewhere.
+    }
+    else if (match_option(option, "nowarning", "W", 6)) {
       ap_options.warning_msg = false;
     }
-    else if (option == "--copyright" || option == "-C") {
+    else if (match_option(option, "copyright", "C", 4)) {
       output_copyright();
       exit(EXIT_SUCCESS);
     }
-    else if (option == "--message" || option == "-M") {
+    else if (match_option(option, "message", "M", 4)) {
       ap_options.info_msg = true;
     }
-    else if (option == "--immutable" || option == "-X") {
+    else if (match_option(option, "immutable", "X", 3)) {
       ap_options.immutable = true;
       stateImmutable       = true;
     }
-    else if (option == "--errors_fatal" || option == "-f") {
+    else if (match_option(option, "errors_fatal", "f", 8)) {
       ap_options.errors_fatal = true;
     }
-    else if (option == "--errors_and_warnings_fatal" || option == "-F") {
+    else if (match_option(option, "errors_and_warnings_fatal", "F", 9)) {
       ap_options.errors_and_warnings_fatal = true;
       ap_options.errors_fatal              = true;
     }
-    else if (option == "--require_defined" || option == "-R") {
+    else if (match_option(option, "require_defined", "R", 3)) {
       ap_options.require_defined = true;
     }
-    else if (option == "--trace" || option == "-t") {
+    else if (match_option(option, "trace", "t", 3)) {
       ap_options.trace_parsing = true;
     }
-    else if (option == "--interactive" || option == "-i") {
+    else if (match_option(option, "interactive", "i", 3)) {
       ap_options.interactive = true;
     }
-    else if (option == "--one_based_index" || option == "-1") {
+    else if (match_option(option, "one_based_index", "1", 3)) {
       ap_options.one_based_index = true;
     }
-    else if (option == "--exit_on" || option == "-e") {
+    else if (match_option(option, "exit_on", "e", 3)) {
       ap_options.end_on_exit = true;
     }
-    else if (option.find("--info") != std::string::npos) {
+    else if (match_option(option, "info", "", 4)) {
       std::string value = get_value(option, optional_value);
       ret_value         = value == optional_value ? 1 : 0;
 
-      auto info = open_file(value, "w");
-      if (info != nullptr) {
-        set_error_streams(nullptr, nullptr, info);
+      if (!value.empty()) {
+        auto do_info = open_file(value, "w");
+        if (do_info != nullptr) {
+          set_error_streams(nullptr, nullptr, do_info, false, false, true);
+        }
       }
     }
-    else if (option.find("--include") != std::string::npos || (option[1] == 'I')) {
+    else if (match_option(option, "include", "I", 3)) {
       std::string value = get_value(option, optional_value);
       ret_value         = value == optional_value ? 1 : 0;
 
@@ -509,11 +565,11 @@ namespace SEAMS {
         ap_options.include_file = value;
       }
     }
-    else if (option == "--keep_history" || option == "-k") {
+    else if (match_option(option, "keep_history", "k", 4)) {
       ap_options.keep_history = true;
     }
 
-    else if (option.find("--comment") != std::string::npos || (option[1] == 'c')) {
+    else if (match_option(option, "comment", "", 3) || (option[1] == 'c')) {
       std::string comment;
       // In short version, do not require equal sign (-c# -c// )
       if (option[1] == 'c' && option.length() > 2 && option[2] != '=') {
@@ -528,53 +584,84 @@ namespace SEAMS {
         ptr->value.svar = comment;
       }
     }
+    else if (match_option(option, "full_precision", "p", 3)) {
+      symrec *ptr = getsym("_FORMAT");
+      if (ptr != nullptr) {
+        ptr->value.svar = "";
+      }
+    }
+    else if (match_option(option, "legacy_output_format", "L", 6)) {
+      symrec *ptr = getsym("_FORMAT");
+      if (ptr != nullptr) {
+        ptr->value.svar = "%.10g";
+      }
+    }
 
-    else if (option == "--help" || option == "-h") {
-      std::cerr
-          << "\nAprepro version " << version() << "\n"
-          << "\nUsage: aprepro [options] [-I path] [-c char] [var=val] [filein] [fileout]\n"
-          << "  --debug or -d: Dump all variables, debug loops/if/endif and keep temporary files\n"
-          << "       --dumpvars or -D: Dump all variables at end of run        \n"
-          << "  --dumpvars_json or -J: Dump all variables at end of run in json format\n"
-          << "        --version or -v: Print version number to stderr          \n"
-          << "      --immutable or -X: All variables are immutable--cannot be modified\n"
-          << "   --errors_fatal or -f: Exit program with nonzero status if errors are "
-             "encountered\n"
-          << " --errors_and_warnings_fatal or -F: Exit program with nonzero status if "
-             "warnings are encountered\n"
-          << "--require_defined or -R: Treat undefined variable warnings as fatal\n"
-          << "--one_based_index or -1: Array indexing is one-based (default = zero-based)\n"
-          << "    --interactive or -i: Interactive use, no buffering           \n"
-          << "    --include=P or -I=P: Include file or include path            \n"
-          << "                       : If P is path, then optionally prepended to all include "
-             "filenames\n"
-          << "                       : If P is file, then processed before processing input file\n"
-          << "                       : variables defined in P will be immutable.\n"
-          << "        --exit_on or -e: End when 'Exit|EXIT|exit' entered       \n"
-          << "           --help or -h: Print this list                         \n"
-          << "        --message or -M: Print INFO messages                     \n"
-          << "            --info=file: Output INFO messages (e.g. DUMP() output) to file.\n"
-          << "      --nowarning or -W: Do not print WARN messages              \n"
-          << "  --comment=char or -c=char: Change comment character to 'char'  \n"
-          << "      --copyright or -C: Print copyright message                 \n"
-          << "   --keep_history or -k: Keep a history of aprepro substitutions.\n"
-          << "                         (not for general interactive use)       \n"
-          << "          --quiet or -q: Do not print the header output line     \n"
-          << "                var=val: Assign value 'val' to variable 'var'    \n"
-          << "                         Use var=\\\"sval\\\" for a string variable\n\n"
-          << "\tUnits Systems: si, cgs, cgs-ev, shock, swap, ft-lbf-s, ft-lbm-s, in-lbf-s\n"
-          << "\tEnter {DUMP()} for list of user-defined variables\n"
-          << "\tEnter {DUMP_FUNC()} for list of functions recognized by aprepro\n"
-          << "\tEnter {DUMP_PREVAR()} for list of predefined variables in aprepro\n\n"
-          << "\tDocumentation: "
-             "https://sandialabs.github.io/seacas-docs/sphinx/html/index.html#aprepro\n\n"
-          << "\t->->-> Send email to gdsjaar@sandia.gov for aprepro support.\n\n";
+    else if (match_option(option, "help", "h", 3)) {
+      fmt::print(stderr,
+                 "\nAprepro version {}\n\n"
+                 "\tUsage: aprepro [options] [-I path] [-c char] [var=val] [filein] [fileout]\n\n",
+                 version());
+
+      format_option("            --debug", "-d",
+                    "Dump all variables, debug loops/if/endif and keep temporary files");
+      format_option("         --dumpvars", "-D", "Dump all variables at end of run        ");
+      format_option("    --dumpvars_json", "-J", "Dump all variables at end of run in json format");
+      format_option("          --version", "-v", "Print version number to stderr          ");
+      format_option("        --immutable", "-X", "All variables are immutable--cannot be modified");
+      format_option("     --errors_fatal", "-f",
+                    "Exit program with nonzero status if errors are encountered");
+      format_option("--errors_and_warnings_fatal", "-F",
+                    "Exit program with nonzero status if warnings are encountered");
+      format_option("  --require_defined", "-R", "Treat undefined variable warnings as fatal");
+      format_option("  --one_based_index", "-1",
+                    "Array indexing is one-based (default = zero-based)");
+      format_option("      --interactive", "-i", "Interactive use, no buffering           ");
+      format_option("      --include=P", "-I=P", "Include file or include path            ");
+      format_option("                         ", "",
+                    "If P is path, then optionally prepended to all include filenames");
+      format_option("                         ", "",
+                    "If P is file, then processed before processing input file");
+      format_option("                         ", "", "variables defined in P will be immutable.");
+      format_option("          --exit_on", "-e", "End when 'Exit|EXIT|exit' entered       ");
+      format_option("             --help", "-h", "Print this list                         ");
+      format_option("          --message", "-M", "Print INFO messages                     ");
+      format_option("              --info=file", "",
+                    "Output INFO messages (e.g. DUMP() output) to file.");
+      format_option("        --nowarning", "-W", "Do not print WARN messages              ");
+      format_option("--comment=char", "-c=char", "Change comment character to 'char'  ");
+      format_option("   --full_precision", "-p",
+                    "Floating point output uses as many digits as needed. [default]");
+      format_option("--legacy_output_format", "-L",
+                    "Floating point output uses the legacy `%.10g` format.");
+      format_option("        --copyright", "-C", "Print copyright message                 ");
+      format_option("     --keep_history", "-k", "Keep a history of aprepro substitutions.");
+      format_option("                         ", "", "(not for general interactive use)       ");
+      format_option("            --quiet", "-q", "Do not print the header output line     ");
+      format_option("                  var=val", "", "Assign value 'val' to variable 'var'");
+      format_option("                         ", "",
+                    "Use var=\\\"sval\\\" for a string variable. 'var' will be immutable.\n");
+      fmt::print(stderr,
+                 "\tUnits Systems: si, cgs, cgs-ev, shock, swap, ft-lbf-s, ft-lbm-s, in-lbf-s\n");
+      fmt::print(stderr, "\tEnter {} for list of user-defined variables\n",
+                 fmt::styled("{{DUMP()}}", fmt::emphasis::bold));
+      fmt::print(stderr, "\tEnter {} for list of functions recognized by aprepro\n",
+                 fmt::styled("{{DUMP_FUNC()}}", fmt::emphasis::bold));
+      fmt::print(stderr, "\tEnter {} for list of predefined variables in aprepro\n\n",
+                 fmt::styled("{{DUMP_PREVAR()}}", fmt::emphasis::bold));
+      fmt::print(stderr,
+                 "\tDocumentation: "
+                 "https://sandialabs.github.io/seacas-docs/sphinx/html/index.html#aprepro\n\n"
+                 "\t->->-> Send email to gdsjaar@sandia.gov for aprepro support.\n\n");
       exit(EXIT_SUCCESS);
     }
+    else {
+      warning("Unrecgonized option '" + option + "'.  This option will be ignored.\n", false);
+    }
     return ret_value;
   }
 
-  array *Aprepro::make_array(int r, int c)
+  array *Aprepro::make_array(size_t r, size_t c)
   {
     auto ptr = new array(r, c);
     array_allocations.push_back(ptr);
@@ -724,8 +811,8 @@ namespace SEAMS {
     for (const auto &ptr : get_sorted_sym_table()) {
       if (!ptr->isInternal) {
         if (ptr->type == Parser::token::VAR || ptr->type == Parser::token::IMMVAR) {
-          (*infoStream) << (first ? "\"" : ",\n\"") << ptr->name << "\": " << std::setprecision(10)
-                        << ptr->value.var;
+          fmt::print((*infoStream), "{}{}\": {}", (first ? "\"" : ",\n\""), ptr->name,
+                     ptr->value.var);
           first = false;
         }
         else if (ptr->type == Parser::token::UNDVAR) {
@@ -759,13 +846,12 @@ namespace SEAMS {
         if (spre.empty() || ptr->name.find(spre) != std::string::npos) {
           if (doInternal == ptr->isInternal) {
             if (ptr->type == Parser::token::VAR) {
-              (*infoStream) << comment << "  {" << std::left << std::setw(width) << ptr->name
-                            << "\t= " << std::setprecision(10) << ptr->value.var << "}" << '\n';
+              fmt::print((*infoStream), "{}  {{{:<{}}\t= {}}}\n", comment, ptr->name, width,
+                         ptr->value.var);
             }
             else if (ptr->type == Parser::token::IMMVAR) {
-              (*infoStream) << comment << "  {" << std::left << std::setw(width) << ptr->name
-                            << "\t= " << std::setprecision(10) << ptr->value.var << "} (immutable)"
-                            << '\n';
+              fmt::print((*infoStream), "{}  {{{:<{}}\t= {}}} (immutable)\n", comment, ptr->name,
+                         width, ptr->value.var);
             }
             else if (ptr->type == Parser::token::SVAR) {
               if (strchr(ptr->value.svar.c_str(), '\n') != nullptr ||
@@ -802,33 +888,35 @@ namespace SEAMS {
     else if (type == Parser::token::FNCT || type == Parser::token::SFNCT ||
              type == Parser::token::AFNCT) {
       int fwidth = 20; // controls spacing/padding for the function names
-      (*infoStream) << trmclr::blue << "\nFunctions returning double:" << trmclr::normal << '\n';
+      fmt::print(*infoStream, "{}",
+                 fmt::styled("\nFunctions returning double:\n", fmt::fg(fmt::color::cyan)));
       for (const auto &ptr : get_sorted_sym_table()) {
         if (spre.empty() || ptr->name.find(spre) != std::string::npos) {
           if (ptr->type == Parser::token::FNCT) {
-            (*infoStream) << std::left << trmclr::green << std::setw(fwidth) << ptr->syntax
-                          << trmclr::normal << ":  " << ptr->info << '\n';
+            fmt::print(*infoStream, "{:<{}}:  {}\n",
+                       fmt::styled(ptr->syntax, fmt::fg(fmt::color::lime)), fwidth, ptr->info);
           }
         }
       }
 
-      (*infoStream) << trmclr::blue << trmclr::blue
-                    << "\nFunctions returning string:" << trmclr::normal << '\n';
+      fmt::print(*infoStream, "{}",
+                 fmt::styled("\nFunctions returning string:\n", fmt::fg(fmt::color::cyan)));
       for (const auto &ptr : get_sorted_sym_table()) {
         if (spre.empty() || ptr->name.find(spre) != std::string::npos) {
           if (ptr->type == Parser::token::SFNCT) {
-            (*infoStream) << std::left << trmclr::green << std::setw(fwidth) << ptr->syntax
-                          << trmclr::normal << ":  " << ptr->info << '\n';
+            fmt::print(*infoStream, "{:<{}}:  {}\n",
+                       fmt::styled(ptr->syntax, fmt::fg(fmt::color::lime)), fwidth, ptr->info);
           }
         }
       }
 
-      (*infoStream) << trmclr::blue << "\nFunctions returning array:" << trmclr::normal << '\n';
+      fmt::print(*infoStream, "{}",
+                 fmt::styled("\nFunctions returning array:\n", fmt::fg(fmt::color::cyan)));
       for (const auto &ptr : get_sorted_sym_table()) {
         if (spre.empty() || ptr->name.find(spre) != std::string::npos) {
           if (ptr->type == Parser::token::AFNCT) {
-            (*infoStream) << std::left << trmclr::green << std::setw(fwidth) << ptr->syntax
-                          << trmclr::normal << ":  " << ptr->info << '\n';
+            fmt::print(*infoStream, "{:<{}}:  {}\n",
+                       fmt::styled(ptr->syntax, fmt::fg(fmt::color::lime)), fwidth, ptr->info);
           }
         }
       }
