@@ -1,5 +1,5 @@
 /*
- * Copyright(C) 1999-2020, 2022 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2020, 2022, 2023 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
@@ -174,39 +174,6 @@ void lanczos_SO(struct vtx_data **A,            /* sparse matrix in row linked l
   int               ritzval_flag;             /* status flag for get_ritzvals() */
   int               memory_ok;                /* True until lanczos runs out of memory */
 
-  struct orthlink *makeorthlnk();             /* makes space for new entry in orthog. set */
-  struct scanlink *mkscanlist();              /* makes initial scan list for min ritz vecs */
-  double          *mkvec(int nl, int nh);     /* allocates space for a vector, dies if problem */
-  double          *mkvec_ret(int nl, int nh); /* allocates space for a vector, returns error code */
-  double dot(double *vec1, int beg, int end, double *vec2); /* standard dot product routine */
-  double ch_norm(double *vec, int beg, int end);            /* vector norm */
-  double Tevec(double *, double *, int, double,
-               double *);    /* calc eigenvector of T by linear recurrence */
-  double checkeig();         /* calculate residual of eigenvector of A */
-  double lanc_seconds(void); /* switcheable timer */
-                             /* free allocated memory safely */
-  int  lanpause();           /* figure when to pause Lanczos iteration */
-  int  get_ritzvals();       /* compute eigenvalues of T */
-  void assign();             /* generate a set assignment from eigenvectors */
-  void setvec();             /* initialize a vector */
-  void vecscale();           /* scale a vector */
-  void splarax();            /* matrix vector multiply */
-  void update(double *vec1, int beg, int end, double *vec2, double fac,
-              double *vec3);     /* add a scalar multiple of a vector to another */
-  void sorthog();                /* orthogonalize a vector against a list of others */
-  void bail();                   /* our exit routine */
-  void scanmin();                /* find small values in vector, store in linked list */
-  void frvec(double *v, int nl); /* free vector */
-  void scadd();                  /* add scalar multiple of vector to another */
-  void orthog1();                /* efficiently orthogonalize against vector of ones */
-  void vecran();                 /* fill vector with random entries */
-  void solistout();              /* print out orthogonalization list */
-  void doubleout();              /* print a double precision number */
-  void orthogvec();              /* orthogonalize one vector against another */
-  void warnings();               /* post various warnings about computation */
-  void mkeigvecs();              /* assemble eigenvectors */
-  void strout(char *msg);        /* print string to screen and output file */
-
   if (DEBUG_TRACE > 0) {
     printf("<Entering lanczos_so>\n");
   }
