@@ -1,4 +1,4 @@
-// Copyright(C) 1999-2020, 2022 National Technology & Engineering Solutions
+// Copyright(C) 1999-2020, 2022, 2023 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 
@@ -22,7 +22,13 @@
 #include <functional>
 #include <stdexcept>
 #include <numeric>
+#if defined(_WIN32) && !defined(__MINGW32__)
+#include <string.h>
+#define strcasecmp _stricmp
+#define strncasecmp _strnicmp
+#else
 #include <strings.h>
+#endif
 
 #include "Iotm_TextMeshFuncs.h"
 #include "Iotm_TextMeshDataTypes.h"
@@ -94,7 +100,7 @@ namespace Iotm {
       struct FaceConnection
       {
         FaceConnection()
-            : thisSide(INVALID_SIDE), thatElement(INVALID_INDEX), thatSide(INVALID_SIDE){};
+            : thisSide(INVALID_SIDE), thatElement(INVALID_INDEX), thatSide(INVALID_SIDE) {};
 
         FaceConnection(int thisSide_, IndexType otherElement_, int otherSide_)
             : thisSide(thisSide_), thatElement(otherElement_), thatSide(otherSide_)
@@ -134,7 +140,10 @@ namespace Iotm {
       struct FaceConnections
       {
         FaceConnections() : numSides(0u) {}
-        FaceConnections(unsigned numSides_) : numSides(numSides_), sideReference(numSides_, 0) {}
+        explicit FaceConnections(unsigned numSides_)
+            : numSides(numSides_), sideReference(numSides_, 0)
+        {
+        }
 
         bool has_any_connection(int thisSide) const
         {
@@ -242,8 +251,8 @@ namespace Iotm {
           for (const auto &entry : iter.second.connections) {
             out << "\tConnected on side: " << entry.thisSide
                 << " to element: " << elemDataVec[entry.thatElement].identifier << " {"
-                << elemDataVec[entry.thatElement].topology << "}"
-                << " and side: " << entry.thatSide << std::endl;
+                << elemDataVec[entry.thatElement].topology << "}" << " and side: " << entry.thatSide
+                << std::endl;
           }
         }
       }
@@ -361,7 +370,8 @@ namespace Iotm {
         std::vector<EntityId> sideNodes =
             get_sorted_side_nodes(adjacency.elementIndex, adjacency.side);
 
-        for (int otherSide = 1; otherSide <= get_element_topology(neighborElementIndex).num_sides();
+        for (int otherSide = 1;
+             otherSide <= get_element_topology(neighborElementIndex).num_face_sides();
              ++otherSide) {
           std::vector<EntityId> otherSideNodes =
               get_sorted_side_nodes(neighborElementIndex, otherSide);
@@ -409,7 +419,7 @@ namespace Iotm {
       std::pair<bool, PermutationType> get_permutation(const Topology              &topology,
                                                        const std::vector<EntityId> &controlNodes,
                                                        const std::vector<EntityId> &permutedNodes,
-                                                       unsigned                     numPermutations)
+                                                       PermutationType              numPermutations)
       {
         PermutationType permutation = Topology::InvalidPermutation;
         bool            equivalent  = false;
@@ -517,7 +527,7 @@ namespace Iotm {
         if (!reciprocity.empty()) {
           FaceConnections &thisEntry = m_indexGraph[adjacency.elementIndex];
 
-          for (FaceConnection &connectionToThatElement : reciprocity) {
+          for (const FaceConnection &connectionToThatElement : reciprocity) {
             IndexType thatIndex = connectionToThatElement.thatElement;
             int       thatSide  = connectionToThatElement.thatSide;
 
@@ -735,7 +745,7 @@ namespace Iotm {
         initialize_side_connectivity_graph(elementIndices);
 
         for (size_t elementIndex : elementIndices) {
-          int numSides = get_element_topology(elementIndex).num_sides();
+          int numSides = get_element_topology(elementIndex).num_face_sides();
           for (int side = 1; side <= numSides; ++side) {
             if (m_indexGraph[elementIndex].sideReference[side - 1] == 0) {
               CurrentAdjacency adjacency(elementIndex, side);
@@ -749,7 +759,7 @@ namespace Iotm {
       {
         for (size_t elementIndex : elementIndices) {
           m_indexGraph[elementIndex] =
-              FaceConnections(get_element_topology(elementIndex).num_sides());
+              FaceConnections(get_element_topology(elementIndex).num_face_sides());
         }
       }
 
