@@ -1,4 +1,4 @@
-// Copyright(C) 1999-2020, 2022 National Technology & Engineering Solutions
+// Copyright(C) 1999-2020, 2022, 2023, 2024, 2025 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
@@ -6,28 +6,28 @@
 
 #pragma once
 
-#include <Ioss_CodeTypes.h>
+#include "Ioss_CodeTypes.h"
 
-#include <Ionit_Initializer.h>
-#include <Ioss_Assembly.h>
-#include <Ioss_CommSet.h>
-#include <Ioss_DBUsage.h>
-#include <Ioss_DatabaseIO.h> // for DatabaseIO
-#include <Ioss_ElementBlock.h>
-#include <Ioss_ElementTopology.h>
-#include <Ioss_EntityType.h>     // for EntityType, etc
-#include <Ioss_Field.h>          // for Field, etc
-#include <Ioss_GroupingEntity.h> // for GroupingEntity
-#include <Ioss_IOFactory.h>      // for IOFactory
-#include <Ioss_MeshType.h>       // for MeshType, etc
-#include <Ioss_NodeBlock.h>
-#include <Ioss_NodeSet.h>
-#include <Ioss_ParallelUtils.h>
-#include <Ioss_PropertyManager.h>
-#include <Ioss_Region.h>
-#include <Ioss_SideBlock.h>
-#include <Ioss_SideSet.h>
-#include <Ioss_StandardElementTypes.h>
+#include "Ionit_Initializer.h"
+#include "Ioss_Assembly.h"
+#include "Ioss_CommSet.h"
+#include "Ioss_DBUsage.h"
+#include "Ioss_DatabaseIO.h" // for DatabaseIO
+#include "Ioss_ElementBlock.h"
+#include "Ioss_ElementTopology.h"
+#include "Ioss_EntityType.h"     // for EntityType, etc
+#include "Ioss_Field.h"          // for Field, etc
+#include "Ioss_GroupingEntity.h" // for GroupingEntity
+#include "Ioss_IOFactory.h"      // for IOFactory
+#include "Ioss_MeshType.h"       // for MeshType, etc
+#include "Ioss_NodeBlock.h"
+#include "Ioss_NodeSet.h"
+#include "Ioss_ParallelUtils.h"
+#include "Ioss_PropertyManager.h"
+#include "Ioss_Region.h"
+#include "Ioss_SideBlock.h"
+#include "Ioss_SideSet.h"
+#include "Ioss_StandardElementTypes.h"
 
 #include <gtest/gtest.h>
 #ifdef SEACAS_HAVE_MPI
@@ -36,11 +36,15 @@
 #include <string>
 #include <unordered_map>
 
-#include <fmt/ostream.h>
-
 #include <memory>
 #include <string>
+#if defined(_WIN32) && !defined(__MINGW32__)
+#include <string.h>
+#define strcasecmp  _stricmp
+#define strncasecmp _strnicmp
+#else
 #include <strings.h>
+#endif
 #include <vector>
 
 #include "text_mesh/Iotm_TextMeshNodeset.h"
@@ -562,8 +566,8 @@ namespace Iotm {
         int    nodesPerElem = topo->number_nodes();
 
         for (size_t i = 0; i < elementCount; ++i) {
-          INT     *conn = &connectivity[i * nodesPerElem];
-          EntityId id   = static_cast<EntityId>(elemIds[i]);
+          INT *conn = &connectivity[i * nodesPerElem];
+          auto id   = static_cast<EntityId>(elemIds[i]);
 
           if (id == elemId) {
             for (int j = 0; j < nodesPerElem; j++) {
@@ -831,8 +835,6 @@ namespace Iotm {
 
         void verify()
         {
-          verify_num_nodes();
-
           for (size_t nodeIndex = 0; nodeIndex < goldNodeIds.size(); nodeIndex++) {
             EntityId nodeId = goldNodeIds[nodeIndex];
             EXPECT_TRUE(is_valid_node(nodeId));
@@ -929,7 +931,7 @@ namespace Iotm {
 
           Ioss::NodeBlock *nb = node_blocks[0];
 
-          size_t node_count = nb->get_property("entity_count").get_int();
+          size_t node_count = nb->entity_count();
 
           EntityIdVector nodeIds = get_node_ids();
 
@@ -950,7 +952,7 @@ namespace Iotm {
 
         const double *get_nodal_coordinates(const EntityId &nodeId) const
         {
-          return (*this)[nodeId].data();
+          return Data((*this)[nodeId]);
         }
 
         void verify_nodal_coordinates(const EntityId &nodeId, const double *goldCoords,
